#
# R companion code to mva_clinres.tex.
# Slides {12,13}
#

source("setup.r")

# Simulate a toy example
n <- 50
X <- replicate(10, rnorm(n))
colnames(X) <- paste("x", 1:10, sep="")
y <- 1.1*X[,1] + 0.8*X[,2] - 0.7*X[,5] + 1.4*X[,6] + rnorm(n)
dfrm <- data.frame(y=y, X)

# Our base model
fm0 <- lm(y ~ 0+x1+x2+x3+x4, data=dfrm)
    
# Build a semi-constrained stepwise regression
fm.step <- step(fm0, scope=list(upper = ~ 0+x1+x2+x3+x4+x5+x6+x7+x8+x9+x10, 
                       lower = ~ 0+x1+x2+x3), trace=FALSE)
summary(fm.step)

# Lasso regression
library(glmnet)
lasso.fit <- glmnet(as.matrix(dfrm[,-1]), dfrm[,1], alpha=1)
enet.cv <- cv.glmnet(as.matrix(dfrm[,-1]), dfrm[,1], alpha=1)
coef(enet.cv$glmnet.fit, s=enet.cv$lambda.1se) # minus one SE trick
normL1 <- apply(abs(lasso.fit$beta), 2, sum)

lasso.df <- melt(data.frame(norm=normL1,
                       t(as.matrix(lasso.fit$beta))), id="norm")
xint <- which(enet.cv$lambda==enet.cv$lambda.1se)
p <- ggplot(lasso.df, aes(norm, value, group=variable, colour=variable)) + geom_line()
p <- p + xlab("Norme L1") + ylab(expression(hat(beta))) + theme_bw2()
my.cols <- ifelse(abs(coef(enet.cv$glmnet.fit, s=enet.cv$lambda.1se)[-1])<1e-9,
                  "lightgrey", "steelblue")
p <- p + geom_vline(xintercept=normL1[xint]) + scale_colour_manual(values=my.cols)
direct.label(p, list("last.points", cex=.6, hjust=-.2))
ggsave("lasso1.pdf", width=5, height=3)

# Regression on principal components
library(pls)
pcr.fit <- pcr(y ~ ., data=dfrm, validation="LOO")

# Ridge regression
enet.cv2 <- cv.glmnet(as.matrix(dfrm[,-1]), dfrm[,1], alpha=0) # ridge

all.res <- data.frame(param=paste("x", 1:10, sep=""),
                      Lasso=coef(enet.cv$glmnet.fit, s=enet.cv$lambda.1se)[-1],
                      Ridge=coef(enet.cv2$glmnet.fit, s=enet.cv$lambda.1se)[-1],
                      PCR=as.numeric(coef(pcr.fit)),
                      OLS=coef(lm(y ~ . -1, data=dfrm)))
all.res$`OLS/AIC` <- NA
all.res[as.numeric(substr(names(coef(fm.step)), 2, 3)),6] <- coef(fm.step)
all.res.df <- melt(all.res)
all.res.df$param <- factor(all.res.df$param, levels=paste("x", 1:10, sep=""), ordered=TRUE)

p <- ggplot(all.res.df, aes(param, value, group=variable, colour=variable)) +
  geom_jitter(position=position_jitter(width=.2)) + scale_colour_discrete(name="Méthode")
p <- p + theme_bw2() + xlab("Terme") + ylab(expression(hat(beta)))
ggsave("benchmark.pdf", width=5, height=3)

# Benchmark of Lasso, Ridge and Elasticnet (RMSE + timing on training sample)
library(caret)
library(doMC)
registerDoMC(2)

trctrl <- trainControl(method="repeatedcv", number=10, repeats=25)
lmFit.lasso <- train(x=dfrm[,-1], y=dfrm[,1], method="lasso",
                     tuneGrid=createGrid("lasso", len=10),
                     trControl=trctrl)
#lmFit.lasso2 <- train(x=dfrm.train[,-1], y=dfrm.train[,1], method="penalized",
#                      tuneGrid=data.frame(.lambda1=seq(1, 10), .lambda2=rep(0, 10)),
#                      trControl=trctrl, trace=FALSE)
lmFit.ridge <- train(x=dfrm[,-1], y=dfrm[,1], method="ridge",
                     tuneGrid=createGrid("ridge", len=10),
                     trControl=trctrl)
lmFit.enet <- train(x=dfrm[,-1], y=dfrm[,1], method="enet",
                     tuneGrid=createGrid("enet", len=10),
                     trControl=trctrl)

predictors(lmFit.lasso)

optim.parm <- lmFit.lasso$bestTune
mod <- lmFit.lasso$finalModel
reg.parms <- scale(betabreaker(mod), FALSE, 1/mod$normx)
xx <- apply(abs(reg.parms), 1, sum)
xx <- xx/max(xx)
rownames(reg.parms) <- xx
lasso.df <- melt(reg.parms)
lasso.df$X3 <- lasso.df$X2
irrelevant.preds <- !(levels(lasso.df$X2) %in% paste("x", 1:10, sep=""))
levels(lasso.df$X3)[irrelevant.preds] <- "x11-x50"
lasso.df$X3 <- factor(lasso.df$X3, levels=c(paste("x", 1:10, sep=""), "x11-x50"))

library(directlabels)
p <- ggplot(lasso.df, aes(X1, value, group=X2, colour=X3)) + geom_line()
p <- p + opts(legend.position="none") +
  theme_bw2() + ylab("Standardized Coefficients") +
  xlab(expression(group("|",beta,"|")/max(group("|",beta,"|"))))

direct.label(p, list("last.qp", cex=.6, hjust=-0.1)) 
ggsave("lasso.pdf", width=5, height=3)

# Timing and precision
resamps <- resamples(list(lasso=lmFit.lasso, ridge=lmFit.ridge,
                          enet=lmFit.enet))
summary(resamps)
resamps$timing

diffs <- diff(resamps, metric = "RMSE")
summary(diffs)
