#
# R companion code to mva_clinres.tex.
# Slides {15,21}
#

source("setup.r")

# Simulate some data
library(mvtnorm)
n <- 80
Sigma <- matrix(c(1,-.5,-.5,0,
                  -.5,1,.5,-.5,
                  -.5,.5,1,-.5,
                  0 ,-.5,-.5,1), nc=4)
dat <- mvrnorm(n, rep(0, 4), Sigma)
dat <- cbind(dat, replicate(40, rnorm(n, 0, 1)))
colnames(dat) <- c("y", paste("x", 1:(ncol(dat)-1), sep=""))

# Elasticnet regression
library(glmnet)
enet.cv <- cv.glmnet(x=dat[,-1], y=dat[,1])
reg.parms <- as.numeric(coef(enet.cv$glmnet.fit, s=enet.cv$lambda.1se)[-1])

# Univariate screening tests
pval <- numeric(43)
for (i in 2:44)
  pval[i-1] <- cor.test(dat[,1], dat[,i])$p.value

pval.df <- data.frame(term=colnames(dat)[2:44],
                      unadj.p=log10(pval),
                      bonf.p=log10(p.adjust(pval, "bonf")),
                      fdr.p=log10(p.adjust(pval, "fdr")))

# Use built-in graphics from glmnet (don't have time to rewrite with ggplot2!)
library(Cairo)
cairo_pdf("enet_cv.pdf", width=10, height=5)
par(mfrow=c(1, 2), las=1)
plot(enet.cv, xlab=expression(log(lambda)), ylab="MSE")

enet.fit <- glmnet(dat[,-1], dat[,1], lambda=enet.cv$lambda)
plot(enet.fit, xvar="lambda", ylab=expression(hat(beta)),
     col=ifelse(abs(reg.parms)<1e-9, "lightgrey", "steelblue"),
     xlab=expression(log(lambda)))
abline(v=log(enet.cv$lambda.1se))
text(log(enet.cv$lambda.1se), reg.parms[reg.parms!=0],
     paste("x", which(reg.parms!=0), sep=""), cex=.8, pos=3,
     col="steelblue")
dev.off()

fm <- as.formula(paste("y ~", paste(paste("x", which(reg.parms!=0), sep=""), collapse="+")))
summary(lm(fm, as.data.frame(dat)))

# Compare Elasticnet and RF with cross-validation (using full sample)
library(caret)
library(doMC)
registerDoMC(2)

trctrl <- trainControl(method="boot", number=100)
lmFit.enet <- train(x=dat[,-1], y=dat[,1], method="glmnet",
                    trControl=trctrl,
                    tuneGrid=createGrid("glmnet", len=10))
plot(lmFit.enet$finalModel)
summary(lmFit.enet$resample[,3:4])

rfFit <- train(x=dat[,-1], y=dat[,1], method="rf",
               importance=TRUE, trControl=trctrl,
               tuneGrid=createGrid("rf", len=10, data=dat[,-1]))


# Timing and accuracy
resamps <- resamples(list(enet=lmFit.enet, rf=rfFit))
summary(resamps)
resamps$timing

# Fitted values on training sample
pool.res <- data.frame(y=dat[,1],
                       RF=predict(rfFit, dat[,-1]),
                       Elasticnet=predict(lmFit.enet, dat[,-1]))
pool.res.df <- melt(pool.res, id="y")
names(pool.res.df)[2] <- "Model"
p <- ggplot(pool.res.df, aes(y, value, group=Model, colour=Model)) +
  geom_point(alpha=.6) + stat_smooth(method="lm", se=FALSE) + geom_abline()
p <- p + theme_bw2() + labs(x="y", y=expression(tilde(y)))
ggsave("allfit.pdf", width=4, height=3)


lmFit.enet$finalModel$beta[,20]
rfFit$finalModel$importance[,1]

# Variable importance
vi.rf.unsc <- varImp(rfFit, scale=FALSE)
vi.rf.sc <- varImp(rfFit, scale=TRUE)
vi.enet.sc <- varImp(lmFit.enet, scale=TRUE)
vi.enet.unsc <- varImp(lmFit.enet, scale=FALSE)

res <- cbind(vi.rf.unsc$importance,
             vi.rf.sc$importance,
             vi.enet.unsc$importance,
             vi.enet.sc$importance)
res <- res[do.call(order, res),]
res <- res[39:43,]
res[seq(5,1,by=-1),]
