#
# R companion code to mva_clinres.tex.
# Slides {4-7,11}
#

source("setup.r")

# Some simulated data, inspired from:
# Li and Xu, High-dimensional data analysis in cancer research, Springer 2009
# Chap. 3, p. 45.
f <- function(x) sin(sqrt(2*pi*x))
n <- 1000
x <- runif(n, 0, 2*pi)
sigma <- rnorm(n, 0, 0.25)
y <- f(x) + sigma
dfrm <- data.frame(x, y)

p <- ggplot(data=dfrm, aes(x, y)) +
  geom_point(alpha=.7, colour="lightgoldenrod3")
p <- p + stat_function(fun=f, lwd=1, colour="steelblue") +
  scale_x_continuous(limits=c(0, 6.5))

library(splines)

# Select 30 random obs. for training set
idx <- sample(1:n, 30)
# 30 other observations for validation set
idx.test <- sample(1:n, 30)

# Fit BS with 3, 5, and 10 DF
fm1 <- lm(y ~ bs(x, df=3), data=dfrm)
fm2 <- lm(y ~ bs(x, df=5), data=dfrm)
fm3 <- lm(y ~ bs(x, df=10), data=dfrm)

# Same models but on a random subsample of 30 obs.
fm1b <- lm(y ~ bs(x, df=3), data=dfrm[idx,])
fm2b <- lm(y ~ bs(x, df=5), data=dfrm[idx,])
fm3b <- lm(y ~ bs(x, df=10), data=dfrm[idx,])

# Get full range x values on [0, 6]
xx <- seq(0, 6, length=1000)

# Predict on full sample (3 and 5 DF only) on full range x
y.bs1 <- predict(fm1, data.frame(x=xx))
y.bs2 <- predict(fm2, data.frame(x=xx))

# Predict on training set on full range x
y.bs1b <- predict(fm1b, data.frame(x=xx))
y.bs2b <- predict(fm2b, data.frame(x=xx))
y.bs3b <- predict(fm3b, data.frame(x=xx))

# Predict on test set on observed x's
# Note: no longer used.
y.bs1c <- predict(fm1b, data.frame(x=dfrm$x[idx.test]))
y.bs2c <- predict(fm2b, data.frame(x=dfrm$x[idx.test]))
y.bs3c <- predict(fm3b, data.frame(x=dfrm$x[idx.test]))

p <- p + theme_bw2()
p + geom_line(data=data.frame(x=xx, y.bs1), aes(x, y.bs1, linetype="3 df")) +
  geom_line(data=data.frame(x=xx, y.bs2), aes(x, y.bs2, linetype="5 df")) +
  scale_linetype_manual("B-Splines", values=c("3 df"=2, "5 df"=3)) +
  opts(legend.position=c(.77,.85),
       legend.key.width = unit(1.5, "cm"))
ggsave("splines.pdf", width=4, height=4)

# Show three fits on the training set
p <- ggplot(data=dfrm[idx,], aes(x, y)) + geom_point(colour="lightgoldenrod3")
p <- p + theme_bw2()
p + geom_line(data=data.frame(x=xx, y.bs1b), aes(x, y.bs1b, linetype="BS-3")) +
  geom_line(data=data.frame(x=xx, y.bs2b), aes(x, y.bs2b, linetype="BS-5")) +
  geom_line(data=data.frame(x=xx, y.bs3b), aes(x, y.bs3b, linetype="BS-10")) +
  scale_linetype_manual("", values=c("BS-3"=1, "BS-5"=2, "BS-10"=3)) +
  opts(legend.position="top", legend.direction="horizontal") + labs(x="", y="")
ggsave("splines2.pdf", width=5, height=3)

# Show four samples of fitted values on external validation sets,
# together with RMSE, when model is estimated on large N with BS-15.
get.rmse <- function(mod, df) {
  yp <- predict(mod, data.frame(x=df$x))
  return(sqrt(mean((yp-df$y)^2)))
}

mod <- lm(y ~ bs(x, df=15), data=dfrm)
bs.full <- predict(mod, data.frame(x=xx))

p <- list()
for (i in 1:4) {
  samp <- sample(1:n, 30)
  bs.fit <- predict(mod, data.frame(x=dfrm$x[samp]))
  rmse <- get.rmse(mod, dfrm[samp,])
  p[[i]] <- ggplot(data=dfrm[samp,], aes(x, y)) +
    geom_point(colour="lightgoldenrod3") +
      scale_y_continuous(limits=c(-2, 2), breaks=seq(-2, 2, by=.5))
  p[[i]] <- p[[i]] +
    geom_line(data=data.frame(x=xx, bs.full), aes(x, bs.full), colour="lightgrey")
  p[[i]] <- p[[i]] + theme_bw2()
  p[[i]] <- p[[i]] +
    geom_point(data=data.frame(x=dfrm$x[samp], bs.fit), aes(x, bs.fit),
               colour="steelblue")
  p[[i]] <- p[[i]] + annotate("text", label=paste("RMSE =", round(rmse, 3)),
                              x=0, y=-2, size=3, hjust=0)
}
pdf("splines3.pdf", width=7, height=7)
do.call(grid.arrange,  p)
dev.off()

# Same as above, but on smaller N.
mod2 <- lm(y ~ bs(x, df=15), data=dfrm[idx,])
bs.full <- predict(mod2, data.frame(x=xx))

p <- list()
for (i in 1:4) {
  samp <- sample(1:n, 30)
  bs.fit <- predict(mod2, data.frame(x=dfrm$x[samp]))
  rmse <- get.rmse(mod2, dfrm[samp,])
  p[[i]] <- ggplot(data=dfrm[samp,], aes(x, y)) +
    geom_point(colour="lightgoldenrod3") +
      scale_y_continuous(limits=c(-2, 2), breaks=seq(-2, 2, by=.5))
  p[[i]] <- p[[i]] +
    geom_line(data=data.frame(x=xx, bs.full), aes(x, bs.full), colour="lightgrey")
  p[[i]] <- p[[i]] + theme_bw2()
  p[[i]] <- p[[i]] +
    geom_point(data=data.frame(x=dfrm$x[samp], bs.fit), aes(x, bs.fit),
               colour="steelblue")
  p[[i]] <- p[[i]] + annotate("text", label=paste("RMSE =", round(rmse, 3)),
                              x=0, y=-2, size=3, hjust=0)
}
pdf("splines4.pdf", width=7, height=7)
do.call(grid.arrange,  p)
dev.off()


# Demonstrate over-fitting on training/test RMSE (without CV)
parms <- seq(1, 12)  # polynom degree
n0 <- 80
subsamp <- dfrm[sample(1:nrow(dfrm), n0),]
train.idx <- sample(1:n0, ceiling(n0/2))
test.idx <- setdiff(1:n0, train.idx)
rmse <- matrix(nr=2, nc=length(parms))

for (p in parms) {
  mod <- lm(y ~ poly(x, p), data=dfrm[train.idx,])
  rmse[1,p] <- get.rmse(mod, dfrm[train.idx,])
  rmse[2,p] <- get.rmse(mod, dfrm[test.idx,])
}

rmse.df <- melt(t(rmse))
names(rmse.df) <- c("Degree", "Sample", "RMSE")
rmse.df$Sample <- factor(rmse.df$Sample, labels=c("Train", "Test"))
p <- ggplot(rmse.df, aes(Degree, RMSE, group=Sample, colour=Sample)) +
  geom_point() + scale_colour_manual("", values=c("Train"="lightgoldenrod3", "Test"="steelblue"))
p + theme_bw2()
ggsave("rmse.pdf", width=5, height=3)

# Short live demo
library(animation)
opt.ani <- ani.options(interval = 0.3, nmax = 300)

# Yet another subsample
dat <- dfrm[sample(1:nrow(dfrm), 80),]

do.it <- function(dfrm, k) {
  require(ggplot2)
  k <- seq(1, k)
  for (d in seq_along(k)) {
    mod <- lm(y ~ poly(x, d), data=dfrm)
    dfrm$pp <- predict(mod)
    dfrm$resid <- abs(resid(mod))
    p <- ggplot(dfrm, aes(x, y)) + geom_point(colour="lightgoldenrod3")
    p <- p + labs(x="", y="") + xlim(0, 6.5) +
      scale_y_continuous(limits=c(-2, 2), breaks=seq(-2, 2, by=.5))
    p <- p + geom_point(aes(x, pp), colour="steelblue") + theme_bw2()
    p <- p + annotate("text", label=paste("k =", d),
                        x=0, y=-2, size=8, hjust=0, vjust=0)
    print(p)
  }
}

# Save video
saveVideo(do.it(dat, 25), video.name="./splines.mov") 

# Reset to default options
ani.options(opt.ani)


# Use regularization instead
opt.ani <- ani.options(interval = 0.1, nmax = 300)
do.it <- function(dfrm, k=25) {
  require(penalized)
  require(ggplot2)
  lam <- c(seq(0, 2, by=.01), seq(2, 30, by=2))
  X <- poly(dfrm$x, k)
  for (l in lam) {
    mod <- penalized(response=dfrm$y, penalized=X, lambda1=0, lambda2=l)
    ## mod <- lm.ridge(y ~ X, data=dfrm, lambda=l)
    ## dfrm$pp <- rowSums(dfrm$x%*%t(coef(mod)))
    dfrm$pp <- predict(mod, X)[,"mu"]
    p <- ggplot(dfrm, aes(x, y)) + geom_point(colour="lightgoldenrod3")
    p <- p + labs(x="", y="") + xlim(0, 6.5) +
      scale_y_continuous(limits=c(-2, 2), breaks=seq(-2, 2, by=.5))
    p <- p + geom_point(aes(x, pp), colour="steelblue") + theme_bw2()
    p <- p + geom_text(data= data.frame(x=0, y=-2,
                         label=paste("lambda",as.character(l), sep="==")),
                       aes(x, y, label=label), parse=TRUE, hjust=0, vjust=0, size=8)
    print(p)
  }
}
saveVideo(do.it(dat), video.name="./ridge.mov") 
ani.options(opt.ani)



